@@ -11,21 +11,27 @@ import pytest
 from _pytest._io.terminalwriter import TerminalWriter
 from _pytest.config import Config, create_terminal_writer
 from _pytest.reports import TestReport
-from pytest import Session
 from strip_ansi import strip_ansi
 
 from pytest_tui.__main__ import tui_launch
 from pytest_tui.html1 import main as tuihtml
-from pytest_tui.utils import (TERMINAL_OUTPUT_FILE, TUI_RESULT_OBJECTS_FILE,
-                              TUI_SECTIONS_FILE, TuiSections, TuiTestResult,
-                              TuiTestResults, errors_section_matcher,
-                              failures_section_matcher, lastline_matcher,
-                              passes_section_matcher,
-                              short_test_summary_matcher,
-                              short_test_summary_test_matcher,
-                              test_session_starts_matcher,
-                              test_session_starts_test_matcher,
-                              warnings_summary_matcher)
+from pytest_tui.utils import (
+    TERMINAL_OUTPUT_FILE,
+    TUI_RESULT_OBJECTS_FILE,
+    TUI_SECTIONS_FILE,
+    TuiSections,
+    TuiTestResult,
+    TuiTestResults,
+    errors_section_matcher,
+    failures_section_matcher,
+    lastline_matcher,
+    passes_section_matcher,
+    short_test_summary_matcher,
+    short_test_summary_test_matcher,
+    test_session_starts_matcher,
+    test_session_starts_test_matcher,
+    warnings_summary_matcher,
+)
 
 # Don't collect tests from any of these files
 collect_ignore = [
@@ -33,32 +39,13 @@ collect_ignore = [
     "plugin.py",
 ]
 
-# Globals to hold Pytest TestReport instances; individual test result objects;
-# and section objects. The global variable approach was chosen becaause the pytest
-# 'config' instance is not accesible in all methods that need it; in particular,
-# pytest_runtest_logstart, where we need to access test results to assign the current
-# value of 'fqtn' to each TestReport instance.
-_tui_reports = []
-_tui_test_results = TuiTestResults()
-_tui_sections = TuiSections()
-_tui_terminal_out = tempfile.TemporaryFile("wb+")
-
-
-def pytest_cmdline_preparse(config, args):
-    """Close global tempfile if pytest is invoked with --version option."""
-    if "--version" in args or "-v" in args:
-        try:
-            _tui_terminal_out.close()
-        except:
-            pass
-
 
 def pytest_addoption(parser) -> None:
     group = parser.getgroup("tui")
     group.addoption(
         "--tui",
         action="store_true",
-        help="Enable the pytest-tui plugin. Both text user interface (TUI) and HTML output are supported.\nRun TUI with console command 'tui'; run HTML report with 'tuih'. Configure autolaunch with 'tuiconf'."
+        help="Enable the pytest-tui plugin. Both text user interface (TUI) and HTML output are supported.\nRun TUI with console command 'tui'; run HTML report with 'tuih'. Configure autolaunch with 'tuiconf'.",
     )
 
 
@@ -99,56 +86,65 @@ def pytest_cmdline_main(config: Config) -> None:
             )
 
 
-def pytest_sessionstart(session: Session) -> None:
-    session.config._tui_sessionstart = (
-        True  # used by tui plugin to indicate first lines of console output
-    )
-    session.config._tui_current_section = "pre_test"  # "" ""
-
-
 def pytest_report_teststatus(report: TestReport, config: Config) -> None:
     """Construct list(s) of individual TestReport instances"""
 
     if hasattr(report, "caplog") and report.caplog:
-        for tui_test_result in _tui_test_results.test_results:
+        for tui_test_result in config._tui_test_results.test_results:
             if tui_test_result.fqtn == report.nodeid:
                 tui_test_result.caplog = report.caplog
 
     if hasattr(report, "capstderr") and report.capstderr:
-        for tui_test_result in _tui_test_results.test_results:
+        for tui_test_result in config._tui_test_results.test_results:
             if tui_test_result.fqtn == report.nodeid:
                 tui_test_result.capstderr = report.capstderr
 
     if hasattr(report, "capstdout") and report.capstdout:
-        for tui_test_result in _tui_test_results.test_results:
+        for tui_test_result in config._tui_test_results.test_results:
             if tui_test_result.fqtn == report.nodeid:
                 tui_test_result.capstdout = report.capstdout
 
     if hasattr(report, "longreprtext") and report.longreprtext:
         add_ansi_to_report(config, report)
-        for tui_test_result in _tui_test_results.test_results:
+        for tui_test_result in config._tui_test_results.test_results:
             if tui_test_result.fqtn == report.nodeid:
                 tui_test_result.longreprtext = report.ansi.val
 
-    _tui_reports.append(report)
+    config._tui_reports.append(report)
 
 
 @pytest.hookimpl()
-def pytest_runtest_logstart(nodeid, location) -> None:
-    for tui_test_result in _tui_test_results.test_results:
-        if tui_test_result.fqtn == nodeid:
+def pytest_runtest_setup(item):
+    for tui_test_result in item.config._tui_test_results.test_results:
+        if tui_test_result.fqtn == item.nodeid:
             tui_test_result.start_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
             break
 
 
 @pytest.hookimpl(trylast=True)
 def pytest_configure(config: Config) -> None:
-    if not (hasattr(config.option, "tui") and config.option.tui):
-        try:
-            _tui_terminal_out.close()
-        except Exception:
-            pass
+    # Don't process any TUI-specific code if the plugin is not enabled
+    if not hasattr(config.option, "tui"):
         return
+    if not config.option.tui:
+        return
+
+    # Initialize Config object items to use throughout rest of test session.
+    # Ideally these would be init'd earlier in the pytest protocol, but it was found that some
+    # custom implementations of pytest frameworks will call pytest.configure() BEFORE other hooks
+    # (like pytest_sessionstart or pytest_load_initial_conftests), so we need to init here.
+    if not hasattr(config, "_tui_sessionstart"):
+        config._tui_sessionstart = True
+    if not hasattr(config, "_tui_current_section"):
+        config._tui_current_section = "pre_test"
+    if not hasattr(config, "_tui_reports"):
+        config._tui_reports = []
+    if not hasattr(config, "_tui_test_results"):
+        config._tui_test_results = TuiTestResults()
+    if not hasattr(config, "_tui_sections"):
+        config._tui_sections = TuiSections()
+    if not hasattr(config, "_tui_terminal_out"):
+        config._tui_terminal_out = tempfile.TemporaryFile("wb+")
 
     # Examine Pytest terminal output to mark different sections of the output.
     # This code is based on the code in pytest's `pastebin.py`.
@@ -189,8 +185,10 @@ def pytest_configure(config: Config) -> None:
                 test_session_starts_test_matcher, s
             ):
                 fqtn = re.search(test_session_starts_test_matcher, s)[1]
-                if fqtn not in [t.fqtn for t in _tui_test_results.test_results]:
-                    _tui_test_results.test_results.append(TuiTestResult(fqtn=fqtn))
+                if fqtn not in [t.fqtn for t in config._tui_test_results.test_results]:
+                    config._tui_test_results.test_results.append(
+                        TuiTestResult(fqtn=fqtn)
+                    )
 
             # If this is an actual test outcome line in the `=== short test summary info ===' section,
             # populate the TuiTestResult's outcome field.
@@ -205,7 +203,7 @@ def pytest_configure(config: Config) -> None:
                     short_test_summary_test_matcher, strip_ansi(s)
                 ).groups()[1]
 
-                for tui_test_result in _tui_test_results.test_results:
+                for tui_test_result in config._tui_test_results.test_results:
                     if tui_test_result.fqtn == fqtn:
                         tui_test_result.outcome = outcome
                         break
@@ -220,11 +218,12 @@ def pytest_configure(config: Config) -> None:
             s_orig = s
             kwargs.pop("flush") if "flush" in kwargs else None
             s_orig = TerminalWriter().markup(s, **kwargs)
-            exec(f"_tui_sections.{config._tui_current_section}.content += s_orig")
+            exec(
+                f"config._tui_sections.{config._tui_current_section}.content += s_orig"
+            )
             if isinstance(s_orig, str):
                 unmarked_up = s_orig.encode("utf-8")
-            global _tui_terminal_out
-            _tui_terminal_out.write(unmarked_up)
+            config._tui_terminal_out.write(unmarked_up)
 
         # Write to both terminal/console and tempfiles
         tr._tw.write = tee_write
@@ -236,7 +235,7 @@ def pytest_unconfigure(config: Config) -> None:
         return
 
     for tui_test_result, test_report in itertools.product(
-        _tui_test_results.test_results, _tui_reports
+        config._tui_test_results.test_results, config._tui_reports
     ):
         if test_report.nodeid == tui_test_result.fqtn:
             tui_test_result.duration += test_report.duration
@@ -248,7 +247,7 @@ def pytest_unconfigure(config: Config) -> None:
     # instead of specifying their test names. This plugin identifies all other test categories
     # (passed, failed, errors, etc.) and populates their fqtns and outcomes with the appropriate
     # values, leaving open one other possibility (Skipped).
-    for tui_test_result in _tui_test_results.test_results:
+    for tui_test_result in config._tui_test_results.test_results:
         if tui_test_result.outcome == "":
             tui_test_result.outcome = "SKIPPED"
 
@@ -256,18 +255,18 @@ def pytest_unconfigure(config: Config) -> None:
 
     # Rewind the temp file containing all the raw ANSI lines sent to the terminal;
     # read its contents;  then close it. Then, write info to file.
-    _tui_terminal_out.seek(0)
-    terminal_out = _tui_terminal_out.read()
-    _tui_terminal_out.close()
+    config._tui_terminal_out.seek(0)
+    terminal_out = config._tui_terminal_out.read()
+    config._tui_terminal_out.close()
     with open(TERMINAL_OUTPUT_FILE, "wb") as file:
         file.write(terminal_out)
 
-    # Pickle the test reult and sections objects to files.
+    # Pickle the test result and sections objects to files.
     file = open(TUI_RESULT_OBJECTS_FILE, "wb")
-    pickle.dump(_tui_test_results, file)
+    pickle.dump(config._tui_test_results, file)
     file.close()
     file = open(TUI_SECTIONS_FILE, "wb")
-    pickle.dump(_tui_sections, file)
+    pickle.dump(config._tui_sections, file)
     file.close()
 
     pytui_launch(config)
